/*-------------------------------------------------------------------------
 Copyright 2006 Olivier Berlanger

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 -------------------------------------------------------------------------*/
package net.sf.xolite;


/**
 * Exception for XML parsing. <br>
 * This exception allow to define the location of the bad tokens that caused the exception in the input XML. This location can
 * be expressed as two int (line, row) or as a plain string.
 */
public class XMLParseException extends Exception {


    private String locationMessage;
    private int line = -1;
    private int column = -1;


    public XMLParseException(String message) {
        super(message);
    }


    public XMLParseException(String message, Throwable cause) {
        super(message, cause);
    }


    @Override
    public String getMessage() {
        String message = super.getMessage();
        String location = getLocationMessage();
        if ((location != null) && !location.equals("")) {
            message += location;
        }
        return message;
    }


    public void setLocation(int newLine, int newColumn) {
        line = newLine;
        column = newColumn;
    }


    public void setLocationMessage(String newMessage) {
        locationMessage = newMessage;
    }


    public String getLocationMessage() {
        if ((locationMessage == null) && (line >= 0)) {
            StringBuilder sb = new StringBuilder();
            sb.append(" at line ");
            sb.append(line);
            if (column >= 0) {
                sb.append(", column ");
                sb.append(column);
            }
            locationMessage = sb.toString();
        }
        return locationMessage;
    }


    public int getLine() {
        return line;
    }


    public int getColumn() {
        return column;
    }


}
