package net.sf.xolite.transform;


import java.io.IOException;

import javax.xml.transform.sax.SAXSource;

import net.sf.xolite.XMLSerializable;
import net.sf.xolite.XMLSerializeException;
import net.sf.xolite.sax.SaxXMLSerializer;

import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.XMLReader;


public class XMLSerializableSource extends SAXSource {


    private InternalXmlReader reader;
    private XMLSerializable root;
    private SaxXMLSerializer serializer;
    /** Dummy source for API compatibility, the real source is the XMLSerializable root object */
    private InputSource dummySource;


    public XMLSerializableSource(XMLSerializable rootSerializable) {
        root = rootSerializable;
        reader = new InternalXmlReader();
        serializer = new SaxXMLSerializer();
    }


    protected SaxXMLSerializer getXMLSerializer() {
        return serializer;
    }


    protected void serialize() throws SAXException {
        try {
            serializer.serialize(root);
        } catch (XMLSerializeException xse) {
            throw new SAXException("Inner serialization exception", xse);
        }
    }


    public void startPrefixMapping(String prefix, String namespaceUri) throws XMLSerializeException {
        serializer.startPrefixMapping(prefix, namespaceUri);
    }


    // ------------------------- SAXSource methods overwrite -----------------------------------------------------


    @Override
    public void setXMLReader(XMLReader xreader) {
        throw new UnsupportedOperationException("Cannot set a reader on TransformationSourceSerializer");
    }


    @Override
    public XMLReader getXMLReader() {
        return reader;
    }


    @Override
    public void setInputSource(InputSource inputSource) {
        throw new UnsupportedOperationException("Cannot set a input source on TransformationSourceSerializer");
    }


    @Override
    public InputSource getInputSource() {
        if (dummySource == null) {
            dummySource = new InputSource(getSystemId());
        }
        return dummySource;
    }


    @Override
    public void setSystemId(String systemId) {
        throw new UnsupportedOperationException("Cannot set a system ID on TransformationSourceSerializer");
    }


    @Override
    public String getSystemId() {
        return "XMLSerializable/" + root.getClass().getName() + "/" + System.identityHashCode(root);
    }


    // ---------------------- Internal XML reader ------------------------------------------------------------


    class InternalXmlReader implements XMLReader {


        public ContentHandler getContentHandler() {
            return getXMLSerializer().getContentHandler();
        }


        public DTDHandler getDTDHandler() {
            return null;
        }


        public EntityResolver getEntityResolver() {
            return null;
        }


        public ErrorHandler getErrorHandler() {
            return null;
        }


        public boolean getFeature(String name) throws SAXNotRecognizedException, SAXNotSupportedException {
            return false;
        }


        public Object getProperty(String name) throws SAXNotRecognizedException, SAXNotSupportedException {
            return null;
        }


        public void parse(InputSource input) throws IOException, SAXException {
            // provided source is ignored
            serialize();
        }


        public void parse(String systemId) throws IOException, SAXException {
            if ((systemId != null) && !systemId.equals(getSystemId()))
                throw new UnsupportedOperationException("Cannot parse external input: " + systemId);
            serialize();
        }


        public void setContentHandler(ContentHandler handler) {
            getXMLSerializer().setContentHandler(handler);
        }


        public void setDTDHandler(DTDHandler newHandler) {
        }


        public void setEntityResolver(EntityResolver newResolver) {
        }


        public void setErrorHandler(ErrorHandler newHandler) {
        }


        public void setFeature(String newName, boolean newValue) throws SAXNotRecognizedException, SAXNotSupportedException {
        }


        public void setProperty(String newName, Object newValue) throws SAXNotRecognizedException, SAXNotSupportedException {
        }

    }
}
